@@ -1,5 +1,5 @@
 package Catalyst::Plugin::AccessLog::Formatter;
-our $VERSION = '0.02';
+our $VERSION = '1.00';
 
 
 # ABSTRACT: Log formatter for Catalyst::Plugin::AccessLog
@@ -40,12 +40,12 @@ sub get_item {
   my ($self, $c, $key, $arg) = @_;
 
   return "[unknown format key $key]" unless exists $items{$key};
-  return $items{$key}->($c, $arg);
+  return $items{$key}->($self, $c, $arg);
 }
 
 sub format_line {
   my ($self, $c) = @_;
-  my $format = $c->config->{'Plugin::AccessLog'}{format};
+  my $format = $self->format;
   my $output = "";
 
   while (1) {
@@ -75,24 +75,48 @@ sub format_line {
 }
 
 
+has 'format' => (
+  is => 'rw',
+  default => '%h %l %u %t "%r" %s %b "%{Referer}i" "%{User-Agent}i"',
+);
+
+
+has 'time_format' => (
+  is => 'rw',
+  default => '%Y-%m-%dT%H:%M:%S',
+);
+
+
+has 'time_zone' => (
+  is => 'rw',
+  default => 'local',
+);
+
+
+has 'hostname_lookups' => (
+  is => 'rw',
+  default => 0,
+);
+
+
 item ['a', 'remote_address'] => sub {
-  return shift->request->address;
+  return $_[1]->request->address;
 };
 
 
 item ['b', 'clf_size'] => sub {
-  return shift->response->content_length || "-";
+  return $_[1]->response->content_length || "-";
 };
 
 
 item ['B', 'size'] => sub {
-  return shift->response->content_length;
+  return $_[1]->response->content_length;
 };
 
 
 item ['h', 'remote_host'] => sub {
-  my $c = shift;
-  if ($c->config->{'Plugin::AccessLog'}{hostname_lookups}) {
+  my ($self, $c) = @_;
+  if ($self->hostname_lookups) {
     return $c->request->hostname;
   } else {
     return $c->request->address;
@@ -101,7 +125,7 @@ item ['h', 'remote_host'] => sub {
 
 
 item ['i', 'header'] => sub {
-  my ($c, $arg) = @_;
+  my ($self, $c, $arg) = @_;
   my $header = $c->req->header($arg);
   return "-" unless defined($header);
   return escape_string($header);
@@ -114,23 +138,23 @@ item 'l' => sub { # for apache compat
 
 
 item ['m', 'method'] => sub {
-  return shift->request->method;
+  return $_[1]->request->method;
 };
 
 
 item ['p', 'port'] => sub {
-  return shift->req->base->port;
+  return $_[1]->req->base->port;
 };
 
 
 item ['r', 'request_line'] => sub { # Mostly for apache's sake
-  my $c = shift;
+  my ($self, $c) = @_;
   return $c->req->method . " /" . $c->req->path . " " . $c->req->protocol;
 };
 
 
 item ['s', 'status'] => sub {
-  return shift->response->status;
+  return $_[1]->response->status;
 };
 
 
@@ -145,50 +169,48 @@ sub _request_start {
 }
 
 item ['t', 'apache_time'] => sub {
-  my ($c, $arg) = @_;
+  my ($self, $c, $arg) = @_;
   return "-" unless $c->use_stats;
-  my $config = $c->config->{'Plugin::AccessLog'};
   my $format = $arg || '[%d/%b/%Y:%H:%M:%S %z]'; # Apache default
   return DateTime->from_epoch(epoch => _request_start($c), 
-    time_zone => $config->{time_zone})->strftime($format);
+    time_zone => $self->time_zone)->strftime($format);
 };
 
 
 item ['time', 'datetime'] => sub {
-  my ($c, $arg) = @_;
+  my ($self, $c, $arg) = @_;
   return "-" unless $c->use_stats;
-  my $config = $c->config->{'Plugin::AccessLog'};
-  my $format = $arg || $config->{time_format};
+  my $format = $arg || $self->time_format;
 
   return DateTime->from_epoch(epoch => _request_start($c),
-    time_zone => $config->{time_zone})->strftime($format);
+    time_zone => $self->time_zone)->strftime($format);
 };
 
 
 item ['u', 'remote_user'] => sub {
-  return shift->request->remote_user || '-';
+  return $_[1]->request->remote_user || '-';
 };
 
 
 item ['V', 'v', 'host_port'] => sub {
-  return shift->request->base->host_port;
+  return $_[1]->request->base->host_port;
 };
 
 
 item 'hostname' => sub {
-  return shift->request->base->host;
+  return $_[1]->request->base->host;
 };
 
 
 # Possibly improvement: use uri_for to absolutize this with base, and then
 # take the path component off of that...
 item ['U', 'path'] => sub {
-  return '/' . shift->request->path;
+  return '/' . $_[1]->request->path;
 };
 
 
 item ['T', 'handle_time'] => sub {
-  my $c = shift;
+  my ($self, $c) = @_;
   if ($c->use_stats) {
     return sprintf "%f", $c->stats->elapsed;
   } else {
@@ -197,6 +219,36 @@ item ['T', 'handle_time'] => sub {
 };
 
 
+item 'action' => sub {
+  return $_[1]->action->reverse;
+};
+
+
+item 'sessionid' => sub {
+  my ($self, $c) = @_;
+  return "-" unless $c->can('sessionid') && defined $c->sessionid;
+  return $c->sessionid;
+};
+
+
+item 'userid' => sub {
+  my ($self, $c) = @_;
+  return "-" unless $c->can('user');
+  return "-" unless $c->user_exists && defined $c->user->id;
+  return $c->user->id;
+};
+
+
+item 'request_count' => sub {
+  $Catalyst::COUNT;
+};
+
+
+item 'pid' => sub {
+  $$
+};
+
+
 no Moose;
 
 1;
@@ -211,7 +263,7 @@ Catalyst::Plugin::AccessLog::Formatter - Log formatter for Catalyst::Plugin::Acc
 
 =head1 VERSION
 
-version 0.02
+version 1.00
 
 =head1 DESCRIPTION
 
@@ -239,6 +291,60 @@ C<%{User-agent}[header]> to get the value of the C<User-agent> header.
 A literal percent-sign can be produced in the output using the escape
 sequence C<%%>.
 
+=head2 Configuration
+
+The following are optional arguments passed to
+C<< Catalyst::Plugin::AccessLog::Formatter->new >>. Ordinarily these values
+will be provided by L<Catalyst::Plugin::AccessLog> from its C<formatter>
+config hash.
+
+=over 4
+
+=item format
+
+B<Default:> C<'%h %l %u %t "%r" %s %b "%{Referer}i" "%{User-Agent}i"'> (Apache
+C<common> log format).
+
+The format string for each line of output. You can use Apache C<LogFormat>
+strings, with a reasonably good level of compatibility, or you can use a
+slightly more readable format. The log format is documented in detail in
+L<Catalyst::Plugin::AccessLog::Formatter>.
+
+
+
+=item time_format
+
+B<Default:> C<'%Y-%m-%dT%H:%M:%S'> (ISO 8601)
+
+The default time format for the C<%t> / C<%[time]> escape. This is a
+C<strftime> format string, which will be provided to L<DateTime>'s
+C<strftime> method.
+
+
+
+=item time_zone
+
+B<Default:> local
+
+The timezone to use when printing times in access logs. This will be passed
+to L<DateTime::TimeZone>'s constructor. Olson timezone names, POSIX TZ
+values, and the keywords C<"local"> and C<"UTC"> are reasonable choices.
+
+
+
+=item hostname_lookups
+
+B<Default:> B<false>
+
+If this option is set to a true value, then the C<%h> /
+C<%[remote_hostname]> escape will resolve the client IP address using
+reverse DNS. This is generally not recommended for reasons of performance
+and security. Equivalent to the Apache option C<HostnameLookups>.
+
+
+
+=back 
+
 =head2 Escapes
 
 =over 4
@@ -330,9 +436,8 @@ The time that the request was received.
 
 While this escape and the C<%[apache_time]> escape both take an optional
 C<strftime> argument, they differ in their default formats. This escape
-defaults to the C<time_format> config option provided to
-C<Catalyst::Plugin::AccessLog>. If that option is not provided, the default
-is ISO 8601.
+defaults to the C<time_format> config option passed to the constructor.
+If that option is not provided, the default is ISO 8601.
 
 
 
@@ -371,6 +476,36 @@ object. Returns a dash C<-> if stats are unavailable.
 
 
 
+=item %[action]
+
+The private path of the Catalyst action that handled the request.
+
+
+
+=item %[sessionid]
+
+The session ID, if there is one, otherwise "-".
+
+
+
+=item %[userid]
+
+The user ID, if Authentication is enabled and a user exists, otherwise "-"
+
+
+
+=item %[request_count]
+
+The request count for the current process, as displayed in debug info.
+
+
+
+=item %[pid]
+
+The process ID of the instance handling the request.
+
+
+
 =back 
 
 
